@@ -26,6 +26,9 @@ OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 PERFORMANCE OF THIS SOFTWARE.
 */
 
+#include "mysql.h"
+#include "mysqld_error.h"
+
 #include "Python.h"
 #if PY_MAJOR_VERSION >= 3
 #define IS_PY3K
@@ -37,17 +40,10 @@ PERFORMANCE OF THIS SOFTWARE.
 
 #include "bytesobject.h"
 #include "structmember.h"
-#include "my_config.h"
-#include "mysql.h"
-#include "mysqld_error.h"
 #include "errmsg.h"
 
 #define MyAlloc(s,t) (s *) t.tp_alloc(&t,0)
-#ifdef IS_PY3K
-# define MyFree(o) Py_TYPE(o)->tp_free((PyObject*)o)
-#else
-# define MyFree(ob) ob->ob_type->tp_free((PyObject *)ob)
-#endif
+#define MyFree(o) Py_TYPE(o)->tp_free((PyObject*)o)
 
 static PyObject *_mysql_MySQLError;
 static PyObject *_mysql_Warning;
@@ -60,7 +56,7 @@ static PyObject *_mysql_IntegrityError;
 static PyObject *_mysql_InternalError; 
 static PyObject *_mysql_ProgrammingError;
 static PyObject *_mysql_NotSupportedError;
- 
+
 typedef struct {
 	PyObject_HEAD
 	MYSQL connection;
@@ -80,17 +76,14 @@ typedef struct {
 	MYSQL_RES *result;
 	int nfields;
 	int use;
+	char has_next;
 	PyObject *converter;
 } _mysql_ResultObject;
 
 extern PyTypeObject _mysql_ResultObject_Type;
 
 static int _mysql_server_init_done = 0;
-#if MYSQL_VERSION_ID >= 40000
 #define check_server_init(x) if (!_mysql_server_init_done) { if (mysql_server_init(0, NULL, NULL)) { _mysql_Exception(NULL); return x; } else { _mysql_server_init_done = 1;} }
-#else
-#define check_server_init(x) if (!_mysql_server_init_done) _mysql_server_init_done = 1
-#endif
 
 #if MYSQL_VERSION_ID >= 50500
 #define HAVE_OPENSSL 1
@@ -118,16 +111,10 @@ _mysql_Exception(_mysql_ConnectionObject *c)
 		return NULL;
 	}
 	merr = mysql_errno(&(c->connection));
-	if (!merr)
+	switch (merr) {
+	case 0:
 		e = _mysql_InterfaceError;
-	else if (merr > CR_MAX_ERROR) {
-		PyTuple_SET_ITEM(t, 0, PyInt_FromLong(-1L));
-		PyTuple_SET_ITEM(t, 1, PyString_FromString("error totally whack"));
-		PyErr_SetObject(_mysql_InterfaceError, t);
-		Py_DECREF(t);
-		return NULL;
-	}
-	else switch (merr) {
+		break;
 	case CR_COMMANDS_OUT_OF_SYNC:
 	case ER_DB_CREATE_EXISTS:
 	case ER_SYNTAX_ERROR:
@@ -185,6 +172,9 @@ _mysql_Exception(_mysql_ConnectionObject *c)
 #endif
 #ifdef ER_CANNOT_ADD_FOREIGN
 	case ER_CANNOT_ADD_FOREIGN:
+#endif
+#ifdef ER_NO_DEFAULT_FOR_FIELD
+	case ER_NO_DEFAULT_FOR_FIELD:
 #endif
 		e = _mysql_IntegrityError;
 		break;
@@ -215,7 +205,7 @@ _mysql_Exception(_mysql_ConnectionObject *c)
 	Py_DECREF(t);
 	return NULL;
 }
-	  
+
 static char _mysql_server_init__doc__[] =
 "Initialize embedded server. If this client is not linked against\n\
 the embedded server library, this function does nothing.\n\
@@ -238,12 +228,11 @@ static PyObject *_mysql_server_init(
 				"already initialized");
 		return NULL;
 	}
-	  
+
 	if (!PyArg_ParseTupleAndKeywords(args, kwargs, "|OO", kwlist,
 					 &cmd_args, &groups))
 		return NULL;
 
-#if MYSQL_VERSION_ID >= 40000
 	if (cmd_args) {
 		if (!PySequence_Check(cmd_args)) {
 			PyErr_SetString(PyExc_TypeError,
@@ -310,7 +299,6 @@ static PyObject *_mysql_server_init(
 		_mysql_Exception(NULL);
 		goto finish;
 	}
-#endif
 	ret = Py_None;
 	Py_INCREF(Py_None);
 	_mysql_server_init_done = 1;
@@ -328,30 +316,26 @@ static PyObject *_mysql_server_end(
 	PyObject *self,
 	PyObject *args) {
 	if (_mysql_server_init_done) {
-#if MYSQL_VERSION_ID >= 40000
 		mysql_server_end();
-#endif
 		_mysql_server_init_done = 0;
 		Py_INCREF(Py_None);
 		return Py_None;
 	}
 	return _mysql_Exception(NULL);
 }
-	 
-#if MYSQL_VERSION_ID >= 32314
+
 static char _mysql_thread_safe__doc__[] =
 "Indicates whether the client is compiled as thread-safe.";
 
 static PyObject *_mysql_thread_safe(
 	PyObject *self,
-	PyObject *args) {
+	PyObject *noargs)
+{
 	PyObject *flag;
-	if (!PyArg_ParseTuple(args, "")) return NULL;
 	check_server_init(NULL);
 	if (!(flag=PyInt_FromLong((long)mysql_thread_safe()))) return NULL;
 	return flag;
 }
-#endif
 
 static char _mysql_ResultObject__doc__[] =
 "result(connection, use=0, converter={}) -- Result set from a query.\n\
@@ -376,8 +360,8 @@ _mysql_ResultObject_Initialize(
 	int n, i;
 	MYSQL_FIELD *fields;
 
-	if (!PyArg_ParseTupleAndKeywords(args, kwargs, "O|iO", kwlist,
-					  &conn, &use, &conv))
+	if (!PyArg_ParseTupleAndKeywords(args, kwargs, "O!|iO", kwlist,
+					 &_mysql_ConnectionObject_Type, &conn, &use, &conv))
 		return -1;
 	if (!conv) {
 		if (!(conv = PyDict_New()))
@@ -395,9 +379,10 @@ _mysql_ResultObject_Initialize(
 	else
 		result = mysql_store_result(&(conn->connection));
 	self->result = result;
+	self->has_next = (char)mysql_more_results(&(conn->connection));
 	Py_END_ALLOW_THREADS ;
 	if (!result) {
-		if (mysql_field_count(&(conn->connection)) > 0) {
+		if (mysql_errno(&(conn->connection))) {
 		    _mysql_Exception(conn);
 		    return -1;
 		}
@@ -496,13 +481,10 @@ static int _mysql_ResultObject_traverse(
 	return 0;
 }
 
-static int _mysql_ResultObject_clear(
-	_mysql_ResultObject *self)
+static int _mysql_ResultObject_clear(_mysql_ResultObject *self)
 {
-	Py_XDECREF(self->converter);
-	self->converter = NULL;
-	Py_XDECREF(self->conn);
-	self->conn = NULL;
+	Py_CLEAR(self->converter);
+	Py_CLEAR(self->conn);
 	return 0;
 }
 
@@ -517,10 +499,12 @@ _mysql_ConnectionObject_Initialize(
 	PyObject *ssl = NULL;
 #if HAVE_OPENSSL
 	char *key = NULL, *cert = NULL, *ca = NULL,
-		*capath = NULL, *cipher = NULL;
+		 *capath = NULL, *cipher = NULL;
+	PyObject *ssl_keepref[5] = {NULL};
+	int n_ssl_keepref = 0;
 #endif
 	char *host = NULL, *user = NULL, *passwd = NULL,
-		*db = NULL, *unix_socket = NULL;
+		 *db = NULL, *unix_socket = NULL;
 	unsigned int port = 0;
 	unsigned int client_flag = 0;
 	static char *kwlist[] = { "host", "user", "passwd", "db", "port",
@@ -544,7 +528,7 @@ _mysql_ConnectionObject_Initialize(
 	char *init_command=NULL,
 	     *read_default_file=NULL,
 	     *read_default_group=NULL;
-	
+
 	self->converter = NULL;
 	self->open = 0;
 	check_server_init(-1);
@@ -573,11 +557,11 @@ _mysql_ConnectionObject_Initialize(
 
 #ifdef IS_PY3K
 #define _stringsuck(d,t,s) {t=PyMapping_GetItemString(s,#d);\
-        if(t){d=PyUnicode_AsUTF8(t);Py_DECREF(t);}\
+        if(t){d=PyUnicode_AsUTF8(t);ssl_keepref[n_ssl_keepref++]=t;}\
         PyErr_Clear();}
 #else
 #define _stringsuck(d,t,s) {t=PyMapping_GetItemString(s,#d);\
-        if(t){d=PyString_AsString(t);Py_DECREF(t);}\
+        if(t){d=PyString_AsString(t);ssl_keepref[n_ssl_keepref++]=t;}\
         PyErr_Clear();}
 #endif
 
@@ -632,9 +616,9 @@ _mysql_ConnectionObject_Initialize(
 		mysql_options(&(self->connection), MYSQL_OPT_LOCAL_INFILE, (char *) &local_infile);
 
 #if HAVE_OPENSSL
-	if (ssl)
-		mysql_ssl_set(&(self->connection),
-			      key, cert, ca, capath, cipher);
+	if (ssl) {
+		mysql_ssl_set(&(self->connection), key, cert, ca, capath, cipher);
+	}
 #endif
 
 	conn = mysql_real_connect(&(self->connection), host, user, passwd, db,
@@ -642,12 +626,22 @@ _mysql_ConnectionObject_Initialize(
 
 	Py_END_ALLOW_THREADS ;
 
+#if HAVE_OPENSSL
+	if (ssl) {
+		int i;
+		for (i=0; i<n_ssl_keepref; i++) {
+			Py_DECREF(ssl_keepref[i]);
+			ssl_keepref[i] = NULL;
+		}
+	}
+#endif
+
 	if (!conn) {
 		_mysql_Exception(self);
 		return -1;
 	}
 
-        /* Internal references to python-land objects */
+	/* Internal references to python-land objects */
 	if (!conv)
 		conv = PyDict_New();
 	else
@@ -728,7 +722,7 @@ _mysql_connect(
 	PyObject *kwargs)
 {
 	_mysql_ConnectionObject *c=NULL;
-	
+
 	c = MyAlloc(_mysql_ConnectionObject, _mysql_ConnectionObject_Type);
 	if (c == NULL) return NULL;
 	if (_mysql_ConnectionObject_Initialize(c, args, kwargs)) {
@@ -772,11 +766,8 @@ static char _mysql_ConnectionObject_close__doc__[] =
 static PyObject *
 _mysql_ConnectionObject_close(
 	_mysql_ConnectionObject *self,
-	PyObject *args)
+	PyObject *noargs)
 {
-	if (args) {
-		if (!PyArg_ParseTuple(args, "")) return NULL;
-	}
 	if (self->open) {
 		Py_BEGIN_ALLOW_THREADS
 		mysql_close(&(self->connection));
@@ -788,8 +779,7 @@ _mysql_ConnectionObject_close(
 		return NULL;
 	}
 	_mysql_ConnectionObject_clear(self);
-	Py_INCREF(Py_None);
-	return Py_None;
+	Py_RETURN_NONE;
 }
 
 static char _mysql_ConnectionObject_affected_rows__doc__ [] =
@@ -800,11 +790,14 @@ Non-standard. Use Cursor.rowcount.\n\
 static PyObject *
 _mysql_ConnectionObject_affected_rows(
 	_mysql_ConnectionObject *self,
-	PyObject *args)
+	PyObject *noargs)
 {
-	if (!PyArg_ParseTuple(args, "")) return NULL;
+	my_ulonglong ret;
 	check_connection(self);
-	return PyLong_FromUnsignedLongLong(mysql_affected_rows(&(self->connection)));
+	ret = mysql_affected_rows(&(self->connection));
+	if (ret == (my_ulonglong)-1)
+		return PyInt_FromLong(-1);
+	return PyLong_FromUnsignedLongLong(ret);
 }
 
 static char _mysql_debug__doc__[] =
@@ -834,10 +827,9 @@ this to work. Non-standard.\n\
 static PyObject *
 _mysql_ConnectionObject_dump_debug_info(
 	_mysql_ConnectionObject *self,
-	PyObject *args)
+	PyObject *noargs)
 {
 	int err;
-	if (!PyArg_ParseTuple(args, "")) return NULL;
 	check_connection(self);
 	Py_BEGIN_ALLOW_THREADS
 	err = mysql_dump_debug_info(&(self->connection));
@@ -858,15 +850,7 @@ _mysql_ConnectionObject_autocommit(
 	int flag, err;
 	if (!PyArg_ParseTuple(args, "i", &flag)) return NULL;
 	Py_BEGIN_ALLOW_THREADS
-#if MYSQL_VERSION_ID >= 40100
 	err = mysql_autocommit(&(self->connection), flag);
-#else
-	{
-		char query[256];
-		snprintf(query, 256, "SET AUTOCOMMIT=%d", flag);
-		err = mysql_query(&(self->connection), query);
-	}
-#endif
 	Py_END_ALLOW_THREADS
 	if (err) return _mysql_Exception(self);
 	Py_INCREF(Py_None);
@@ -893,16 +877,11 @@ static char _mysql_ConnectionObject_commit__doc__[] =
 static PyObject *
 _mysql_ConnectionObject_commit(
 	_mysql_ConnectionObject *self,
-	PyObject *args)
+	PyObject *noargs)
 {
 	int err;
-	if (!PyArg_ParseTuple(args, "")) return NULL;
 	Py_BEGIN_ALLOW_THREADS
-#if MYSQL_VERSION_ID >= 40100
 	err = mysql_commit(&(self->connection));
-#else
-	err = mysql_query(&(self->connection), "COMMIT");
-#endif
 	Py_END_ALLOW_THREADS
 	if (err) return _mysql_Exception(self);
 	Py_INCREF(Py_None);
@@ -915,16 +894,11 @@ static char _mysql_ConnectionObject_rollback__doc__[] =
 static PyObject *
 _mysql_ConnectionObject_rollback(
 	_mysql_ConnectionObject *self,
-	PyObject *args)
+	PyObject *noargs)
 {
 	int err;
-	if (!PyArg_ParseTuple(args, "")) return NULL;
 	Py_BEGIN_ALLOW_THREADS
-#if MYSQL_VERSION_ID >= 40100
 	err = mysql_rollback(&(self->connection));
-#else
-	err = mysql_query(&(self->connection), "ROLLBACK");
-#endif
 	Py_END_ALLOW_THREADS
 	if (err) return _mysql_Exception(self);
 	Py_INCREF(Py_None);
@@ -947,22 +921,16 @@ Non-standard.\n\
 static PyObject *
 _mysql_ConnectionObject_next_result(
 	_mysql_ConnectionObject *self,
-	PyObject *args)
+	PyObject *noargs)
 {
 	int err;
-	if (!PyArg_ParseTuple(args, "")) return NULL;
 	Py_BEGIN_ALLOW_THREADS
-#if MYSQL_VERSION_ID >= 40100
 	err = mysql_next_result(&(self->connection));
-#else
-	err = -1;
-#endif
 	Py_END_ALLOW_THREADS
 	if (err > 0) return _mysql_Exception(self);
 	return PyInt_FromLong(err);
 }		
 
-#if MYSQL_VERSION_ID >= 40100
 
 static char _mysql_ConnectionObject_set_server_option__doc__[] =
 "set_server_option(option) -- Enables or disables an option\n\
@@ -1000,9 +968,8 @@ Non-standard.\n\
 static PyObject *
 _mysql_ConnectionObject_sqlstate(
 	_mysql_ConnectionObject *self,
-	PyObject *args)
+	PyObject *noargs)
 {
-	if (!PyArg_ParseTuple(args, "")) return NULL;
 	return PyString_FromString(mysql_sqlstate(&(self->connection)));
 }
 
@@ -1015,14 +982,11 @@ Non-standard.\n\
 static PyObject *
 _mysql_ConnectionObject_warning_count(
 	_mysql_ConnectionObject *self,
-	PyObject *args)
+	PyObject *noargs)
 {
-	if (!PyArg_ParseTuple(args, "")) return NULL;
 	return PyInt_FromLong(mysql_warning_count(&(self->connection)));
 }		
 
-#endif
-
 static char _mysql_ConnectionObject_errno__doc__[] =
 "Returns the error code for the most recently invoked API function\n\
 that can succeed or fail. A return value of zero means that no error\n\
@@ -1032,9 +996,8 @@ occurred.\n\
 static PyObject *
 _mysql_ConnectionObject_errno(
 	_mysql_ConnectionObject *self,
-	PyObject *args)
+	PyObject *noargs)
 {
-	if (!PyArg_ParseTuple(args, "")) return NULL;
 	check_connection(self);
 	return PyInt_FromLong((long)mysql_errno(&(self->connection)));
 }
@@ -1048,9 +1011,8 @@ occurred.\n\
 static PyObject *
 _mysql_ConnectionObject_error(
 	_mysql_ConnectionObject *self,
-	PyObject *args)
+	PyObject *noargs)
 {
-	if (!PyArg_ParseTuple(args, "")) return NULL;
 	check_connection(self);
 	return PyString_FromString(mysql_error(&(self->connection)));
 }
@@ -1071,22 +1033,23 @@ _mysql_escape_string(
 	PyObject *str;
 	char *in, *out;
 	int len, size;
-	if (!PyArg_ParseTuple(args, "y#:escape_string", &in, &size)) return NULL;
+	if (!PyArg_ParseTuple(args, "s#:escape_string", &in, &size)) return NULL;
 	str = PyBytes_FromStringAndSize((char *) NULL, size*2+1);
 	if (!str) return PyErr_NoMemory();
 	out = PyBytes_AS_STRING(str);
-#if MYSQL_VERSION_ID < 32321
-	len = mysql_escape_string(out, in, size);
-#else
 	check_server_init(NULL);
 
 	if (self && PyModule_Check((PyObject*)self))
 		self = NULL;
-	if (self && self->open)
+	if (self && self->open) {
+#if MYSQL_VERSION_ID >= 50707 && !defined(MARIADB_BASE_VERSION) && !defined(MARIADB_VERSION_ID)
+		len = mysql_real_escape_string_quote(&(self->connection), out, in, size, '\'');
+#else
 		len = mysql_real_escape_string(&(self->connection), out, in, size);
-	else
-		len = mysql_escape_string(out, in, size);
 #endif
+	} else {
+		len = mysql_escape_string(out, in, size);
+	}
 	if (_PyBytes_Resize(&str, len) < 0) return NULL;
 	return (str);
 }
@@ -1135,15 +1098,16 @@ _mysql_string_literal(
 		return PyErr_NoMemory();
 	}
 	out = PyBytes_AS_STRING(str);
-#if MYSQL_VERSION_ID < 32321
-	len = mysql_escape_string(out+1, in, size);
-#else
 	check_server_init(NULL);
-	if (self && self->open)
+	if (self && self->open) {
+#if MYSQL_VERSION_ID >= 50707 && !defined(MARIADB_BASE_VERSION) && !defined(MARIADB_VERSION_ID)
+		len = mysql_real_escape_string_quote(&(self->connection), out+1, in, size, '\'');
+#else
 		len = mysql_real_escape_string(&(self->connection), out+1, in, size);
-	else
-		len = mysql_escape_string(out+1, in, size);
 #endif
+	} else {
+		len = mysql_escape_string(out+1, in, size);
+	}
 	*out = *(out+len+1) = '\'';
 	if (_PyBytes_Resize(&str, len+2) < 0) return NULL;
 	Py_DECREF(s);
@@ -1278,7 +1242,7 @@ _mysql_escape_dict(
 	Py_XDECREF(r);
 	return NULL;
 }
-				
+
 static char _mysql_ResultObject_describe__doc__[] =
 "Returns the sequence of 7-tuples required by the DB-API for\n\
 the Cursor.description attribute.\n\
@@ -1287,12 +1251,11 @@ the Cursor.description attribute.\n\
 static PyObject *
 _mysql_ResultObject_describe(
 	_mysql_ResultObject *self,
-	PyObject *args)
+	PyObject *noargs)
 {
 	PyObject *d;
 	MYSQL_FIELD *fields;
 	unsigned int i, n;
-	if (!PyArg_ParseTuple(args, "")) return NULL;
 	check_result_connection(self);
 	n = mysql_num_fields(self->result);
 	fields = mysql_fetch_fields(self->result);
@@ -1315,7 +1278,7 @@ _mysql_ResultObject_describe(
 	Py_XDECREF(d);
 	return NULL;
 }
-	
+
 static char _mysql_ResultObject_field_flags__doc__[] =
 "Returns a tuple of field flags, one for each column in the result.\n\
 " ;
@@ -1323,12 +1286,11 @@ static char _mysql_ResultObject_field_flags__doc__[] =
 static PyObject *
 _mysql_ResultObject_field_flags(
 	_mysql_ResultObject *self,
-	PyObject *args)
+	PyObject *noargs)
 {
 	PyObject *d;
 	MYSQL_FIELD *fields;
 	unsigned int i, n;
-	if (!PyArg_ParseTuple(args, "")) return NULL;
 	check_result_connection(self);
 	n = mysql_num_fields(self->result);
 	fields = mysql_fetch_fields(self->result);
@@ -1351,9 +1313,9 @@ _mysql_field_to_python(
 	unsigned long length,
 	MYSQL_FIELD *field)
 {
-	int field_type = field->type;
 	PyObject *v;
 #ifdef IS_PY3K
+	int field_type = field->type;
 	// Return bytes for binary and string types.
 	int binary = 0;
 	if (field_type == FIELD_TYPE_TINY_BLOB ||
@@ -1361,13 +1323,14 @@ _mysql_field_to_python(
 		field_type == FIELD_TYPE_LONG_BLOB ||
 		field_type == FIELD_TYPE_BLOB ||
 		field_type == FIELD_TYPE_VAR_STRING ||
-		field_type == FIELD_TYPE_STRING) {
+		field_type == FIELD_TYPE_STRING ||
+		field_type == FIELD_TYPE_GEOMETRY ||
+		field_type == FIELD_TYPE_BIT) {
 			binary = 1;
 	}
 #endif
 	if (rowitem) {
 		if (converter != Py_None) {
-			const char *fmt = "s#";
 			v = PyObject_CallFunction(converter,
 #ifdef IS_PY3K
 						  binary ? "y#" : "s#",
@@ -1508,7 +1471,7 @@ _mysql__fetch_row(
 	int maxrows,
 	_PYFUNC *convert_row)
 {
-	unsigned int i;
+	int i;
 	MYSQL_ROW row;
 
 	for (i = skiprows; i<(skiprows+maxrows); i++) {
@@ -1561,14 +1524,14 @@ _mysql_ResultObject_fetch_row(
 		_mysql_row_to_dict_old
 	};
 	_PYFUNC *convert_row;
-	unsigned int maxrows=1, how=0, skiprows=0, rowsadded;
+	int maxrows=1, how=0, skiprows=0, rowsadded;
 	PyObject *r=NULL;
 
 	if (!PyArg_ParseTupleAndKeywords(args, kwargs, "|ii:fetch_row", kwlist,
 					 &maxrows, &how))
 		return NULL;
 	check_result_connection(self);
-	if (how < 0 || how >= sizeof(row_converters)) {
+	if (how >= (int)sizeof(row_converters)) {
 		PyErr_SetString(PyExc_ValueError, "how out of range");
 		return NULL;
 	}
@@ -1606,8 +1569,6 @@ _mysql_ResultObject_fetch_row(
 	return NULL;
 }
 
-#if MYSQL_VERSION_ID >= 32303
-
 static char _mysql_ConnectionObject_change_user__doc__[] =
 "Changes the user and causes the database specified by db to\n\
 become the default (current) database on the connection\n\
@@ -1646,7 +1607,6 @@ _mysql_ConnectionObject_change_user(
 	Py_INCREF(Py_None);
 	return Py_None;
 }
-#endif
 
 static char _mysql_ConnectionObject_character_set_name__doc__[] =
 "Returns the default character set for the current connection.\n\
@@ -1656,20 +1616,14 @@ Non-standard.\n\
 static PyObject *
 _mysql_ConnectionObject_character_set_name(
 	_mysql_ConnectionObject *self,
-	PyObject *args)
+	PyObject *noargs)
 {
 	const char *s;
-	if (!PyArg_ParseTuple(args, "")) return NULL;
 	check_connection(self);
-#if MYSQL_VERSION_ID >= 32321
 	s = mysql_character_set_name(&(self->connection));
-#else
-	s = "latin1";
-#endif
 	return PyString_FromString(s);
 }
 
-#if MYSQL_VERSION_ID >= 50007
 static char _mysql_ConnectionObject_set_character_set__doc__[] =
 "Sets the default character set for the current connection.\n\
 Non-standard.\n\
@@ -1691,7 +1645,6 @@ _mysql_ConnectionObject_set_character_set(
 	Py_INCREF(Py_None);
 	return Py_None;
 }
-#endif
 
 #if MYSQL_VERSION_ID >= 50010
 static char _mysql_ConnectionObject_get_character_set_info__doc__[] =
@@ -1718,12 +1671,11 @@ Non-standard.\n\
 static PyObject *
 _mysql_ConnectionObject_get_character_set_info(
 	_mysql_ConnectionObject *self,
-	PyObject *args)
+	PyObject *noargs)
 {
 	PyObject *result;
 	MY_CHARSET_INFO cs;
-	
-	if (!PyArg_ParseTuple(args, "")) return NULL;
+
 	check_connection(self);
 	mysql_get_character_set_info(&(self->connection), &cs);
 	if (!(result = PyDict_New())) return NULL;
@@ -1747,9 +1699,8 @@ the client library version.";
 static PyObject *
 _mysql_get_client_info(
 	PyObject *self,
-	PyObject *args)
+	PyObject *noargs)
 {
-	if (!PyArg_ParseTuple(args, "")) return NULL;
 	check_server_init(NULL);
 	return PyString_FromString(mysql_get_client_info());
 }
@@ -1762,9 +1713,8 @@ version. Non-standard.\n\
 static PyObject *
 _mysql_ConnectionObject_get_host_info(
 	_mysql_ConnectionObject *self,
-	PyObject *args)
+	PyObject *noargs)
 {
-	if (!PyArg_ParseTuple(args, "")) return NULL;
 	check_connection(self);
 	return PyString_FromString(mysql_get_host_info(&(self->connection)));
 }
@@ -1777,9 +1727,8 @@ used by the current connection. Non-standard.\n\
 static PyObject *
 _mysql_ConnectionObject_get_proto_info(
 	_mysql_ConnectionObject *self,
-	PyObject *args)
+	PyObject *noargs)
 {
-	if (!PyArg_ParseTuple(args, "")) return NULL;
 	check_connection(self);
 	return PyInt_FromLong((long)mysql_get_proto_info(&(self->connection)));
 }
@@ -1792,9 +1741,8 @@ Non-standard.\n\
 static PyObject *
 _mysql_ConnectionObject_get_server_info(
 	_mysql_ConnectionObject *self,
-	PyObject *args)
+	PyObject *noargs)
 {
-	if (!PyArg_ParseTuple(args, "")) return NULL;
 	check_connection(self);
 	return PyString_FromString(mysql_get_server_info(&(self->connection)));
 }
@@ -1808,10 +1756,9 @@ Cursor.messages.\n\
 static PyObject *
 _mysql_ConnectionObject_info(
 	_mysql_ConnectionObject *self,
-	PyObject *args)
+	PyObject *noargs)
 {
 	const char *s;
-	if (!PyArg_ParseTuple(args, "")) return NULL;
 	check_connection(self);
 	s = mysql_info(&(self->connection));
 	if (s) return PyString_FromString(s);
@@ -1843,10 +1790,9 @@ in the server.\n\
 static PyObject *
 _mysql_ConnectionObject_insert_id(
 	_mysql_ConnectionObject *self,
-	PyObject *args)
+	PyObject *noargs)
 {
 	my_ulonglong r;
-	if (!PyArg_ParseTuple(args, "")) return NULL;
 	check_connection(self);
 	Py_BEGIN_ALLOW_THREADS
 	r = mysql_insert_id(&(self->connection));
@@ -1884,15 +1830,10 @@ on most cursor classes. Use Cursor.rowcount.\n\
 static PyObject *
 _mysql_ConnectionObject_field_count(
 	_mysql_ConnectionObject *self,
-	PyObject *args)
+	PyObject *noargs)
 {
-	if (!PyArg_ParseTuple(args, "")) return NULL;
 	check_connection(self);
-#if MYSQL_VERSION_ID < 32224
-	return PyInt_FromLong((long)mysql_num_fields(&(self->connection)));
-#else
 	return PyInt_FromLong((long)mysql_field_count(&(self->connection)));
-#endif
 }	
 
 static char _mysql_ResultObject_num_fields__doc__[] =
@@ -1901,9 +1842,8 @@ static char _mysql_ResultObject_num_fields__doc__[] =
 static PyObject *
 _mysql_ResultObject_num_fields(
 	_mysql_ResultObject *self,
-	PyObject *args)
+	PyObject *noargs)
 {
-	if (!PyArg_ParseTuple(args, "")) return NULL;
 	check_result_connection(self);
 	return PyInt_FromLong((long)mysql_num_fields(self->result));
 }	
@@ -1917,9 +1857,8 @@ set has been read.\n\
 static PyObject *
 _mysql_ResultObject_num_rows(
 	_mysql_ResultObject *self,
-	PyObject *args)
+	PyObject *noargs)
 {
-	if (!PyArg_ParseTuple(args, "")) return NULL;
 	check_result_connection(self);
 	return PyLong_FromUnsignedLongLong(mysql_num_rows(self->result));
 }	
@@ -1951,7 +1890,10 @@ _mysql_ConnectionObject_ping(
 	int r, reconnect = -1;
 	if (!PyArg_ParseTuple(args, "|I", &reconnect)) return NULL;
 	check_connection(self);
-	if ( reconnect != -1 ) self->connection.reconnect = reconnect;
+	if (reconnect != -1) {
+		my_bool recon = reconnect;
+		mysql_options(&self->connection, MYSQL_OPT_RECONNECT, &recon);
+	}
 	Py_BEGIN_ALLOW_THREADS
 	r = mysql_ping(&(self->connection));
 	Py_END_ALLOW_THREADS
@@ -1973,7 +1915,6 @@ _mysql_ConnectionObject_query(
 {
 	char *query;
 	int len, r;
-	MYSQL *mysql = &(self->connection);
 	if (!PyArg_ParseTuple(args, "s#:query", &query, &len)) return NULL;
 	check_connection(self);
 
@@ -2015,10 +1956,10 @@ static char _mysql_ConnectionObject_read_query_result__doc__[] =
 
 static PyObject *
 _mysql_ConnectionObject_read_query_result(
-	_mysql_ConnectionObject *self)
+	_mysql_ConnectionObject *self,
+	PyObject *noargs)
 {
-	char *query;
-	int len, r;
+	int r;
 	MYSQL *mysql = &(self->connection);
 	check_connection(self);
 
@@ -2067,17 +2008,12 @@ have shutdown privileges. Non-standard.\n\
 static PyObject *
 _mysql_ConnectionObject_shutdown(
 	_mysql_ConnectionObject *self,
-	PyObject *args)
+	PyObject *noargs)
 {
 	int r;
-	if (!PyArg_ParseTuple(args, "")) return NULL;
 	check_connection(self);
 	Py_BEGIN_ALLOW_THREADS
-	r = mysql_shutdown(&(self->connection)
-#if MYSQL_VERSION_ID >= 40103
-		, SHUTDOWN_DEFAULT
-#endif
-		);
+	r = mysql_shutdown(&(self->connection), SHUTDOWN_DEFAULT);
 	Py_END_ALLOW_THREADS
 	if (r) return _mysql_Exception(self);
 	Py_INCREF(Py_None);
@@ -2094,10 +2030,9 @@ questions, reloads, and open tables. Non-standard.\n\
 static PyObject *
 _mysql_ConnectionObject_stat(
 	_mysql_ConnectionObject *self,
-	PyObject *args)
+	PyObject *noargs)
 {
 	const char *s;
-	if (!PyArg_ParseTuple(args, "")) return NULL;
 	check_connection(self);
 	Py_BEGIN_ALLOW_THREADS
 	s = mysql_stat(&(self->connection));
@@ -2115,12 +2050,11 @@ None is returned. Non-standard.\n\
 static PyObject *
 _mysql_ConnectionObject_store_result(
 	_mysql_ConnectionObject *self,
-	PyObject *args)
+	PyObject *noargs)
 {
 	PyObject *arglist=NULL, *kwarglist=NULL, *result=NULL;
 	_mysql_ResultObject *r=NULL;
 
-	if (!PyArg_ParseTuple(args, "")) return NULL;
 	check_connection(self);
 	arglist = Py_BuildValue("(OiO)", self, 0, self->converter);
 	if (!arglist) goto error;
@@ -2156,10 +2090,9 @@ Non-standard.";
 static PyObject *
 _mysql_ConnectionObject_thread_id(
 	_mysql_ConnectionObject *self,
-	PyObject *args)
+	PyObject *noargs)
 {
 	unsigned long pid;
-	if (!PyArg_ParseTuple(args, "")) return NULL;
 	check_connection(self);
 	Py_BEGIN_ALLOW_THREADS
 	pid = mysql_thread_id(&(self->connection));
@@ -2176,12 +2109,11 @@ None is returned. Non-standard.\n\
 static PyObject *
 _mysql_ConnectionObject_use_result(
 	_mysql_ConnectionObject *self,
-	PyObject *args)
+	PyObject *noargs)
 {
 	PyObject *arglist=NULL, *kwarglist=NULL, *result=NULL;
 	_mysql_ResultObject *r=NULL;
 
-	if (!PyArg_ParseTuple(args, "")) return NULL;
 	check_connection(self);
 	arglist = Py_BuildValue("(OiO)", self, 1, self->converter);
 	if (!arglist) return NULL;
@@ -2207,13 +2139,12 @@ static void
 _mysql_ConnectionObject_dealloc(
 	_mysql_ConnectionObject *self)
 {
-	PyObject *o;
-
 	PyObject_GC_UnTrack(self);
 	if (self->open) {
-		o = _mysql_ConnectionObject_close(self, NULL);
-		Py_XDECREF(o);
+		mysql_close(&(self->connection));
+		self->open = 0;
 	}
+	Py_CLEAR(self->converter);
 	MyFree(self);
 }
 
@@ -2274,10 +2205,9 @@ static char _mysql_ResultObject_row_tell__doc__[] =
 static PyObject *
 _mysql_ResultObject_row_tell(
 	_mysql_ResultObject *self,
-	PyObject *args)
+	PyObject *noargs)
 {
 	MYSQL_ROW_OFFSET r;
-	if (!PyArg_ParseTuple(args, "")) return NULL;
 	check_result_connection(self);
 	if (self->use) {
 		PyErr_SetString(_mysql_ProgrammingError,
@@ -2311,7 +2241,7 @@ static PyMethodDef _mysql_ConnectionObject_methods[] = {
 	{
 		"affected_rows",
 		(PyCFunction)_mysql_ConnectionObject_affected_rows,
-		METH_VARARGS,
+		METH_NOARGS,
 		_mysql_ConnectionObject_affected_rows__doc__
 	},
 	{
@@ -2329,22 +2259,21 @@ static PyMethodDef _mysql_ConnectionObject_methods[] = {
 	{
 		"commit",
 		(PyCFunction)_mysql_ConnectionObject_commit,
-		METH_VARARGS,
+		METH_NOARGS,
 		_mysql_ConnectionObject_commit__doc__
 	},
 	{
 		"rollback",
 		(PyCFunction)_mysql_ConnectionObject_rollback,
-		METH_VARARGS,
+		METH_NOARGS,
 		_mysql_ConnectionObject_rollback__doc__
 	},
 	{
 		"next_result",
 		(PyCFunction)_mysql_ConnectionObject_next_result,
-		METH_VARARGS,
+		METH_NOARGS,
 		_mysql_ConnectionObject_next_result__doc__
 	},
-#if MYSQL_VERSION_ID >= 40100
 	{
 		"set_server_option",
 		(PyCFunction)_mysql_ConnectionObject_set_server_option,
@@ -2354,50 +2283,45 @@ static PyMethodDef _mysql_ConnectionObject_methods[] = {
 	{
 		"sqlstate",
 		(PyCFunction)_mysql_ConnectionObject_sqlstate,
-		METH_VARARGS,
+		METH_NOARGS,
 		_mysql_ConnectionObject_sqlstate__doc__
 	},
 	{
 		"warning_count",
 		(PyCFunction)_mysql_ConnectionObject_warning_count,
-		METH_VARARGS,
+		METH_NOARGS,
 		_mysql_ConnectionObject_warning_count__doc__
 	},
-#endif
-#if MYSQL_VERSION_ID >= 32303
 	{
 		"change_user",
 		(PyCFunction)_mysql_ConnectionObject_change_user,
 		METH_VARARGS | METH_KEYWORDS,
 		_mysql_ConnectionObject_change_user__doc__
 	},
-#endif
 	{
 		"character_set_name",
 		(PyCFunction)_mysql_ConnectionObject_character_set_name,
-		METH_VARARGS,
+		METH_NOARGS,
 		_mysql_ConnectionObject_character_set_name__doc__
 	},
-#if MYSQL_VERSION_ID >= 50007
 	{
 		"set_character_set",
 		(PyCFunction)_mysql_ConnectionObject_set_character_set,
 		METH_VARARGS,
 		_mysql_ConnectionObject_set_character_set__doc__
 	},
-#endif
 #if MYSQL_VERSION_ID >= 50010
 	{
 		"get_character_set_info",
 		(PyCFunction)_mysql_ConnectionObject_get_character_set_info,
-		METH_VARARGS,
+		METH_NOARGS,
 		_mysql_ConnectionObject_get_character_set_info__doc__
 	},
 #endif
 	{
 		"close",
 		(PyCFunction)_mysql_ConnectionObject_close,
-		METH_VARARGS,
+		METH_NOARGS,
 		_mysql_ConnectionObject_close__doc__
 	},
 	{
@@ -2409,7 +2333,7 @@ static PyMethodDef _mysql_ConnectionObject_methods[] = {
 	{
 		"dump_debug_info",
 		(PyCFunction)_mysql_ConnectionObject_dump_debug_info,
-		METH_VARARGS,
+		METH_NOARGS,
 		_mysql_ConnectionObject_dump_debug_info__doc__
 	},
 	{
@@ -2427,49 +2351,49 @@ static PyMethodDef _mysql_ConnectionObject_methods[] = {
 	{
 		"error",
 		(PyCFunction)_mysql_ConnectionObject_error,
-		METH_VARARGS,
+		METH_NOARGS,
 		_mysql_ConnectionObject_error__doc__
 	},
 	{
 		"errno",
 		(PyCFunction)_mysql_ConnectionObject_errno,
-		METH_VARARGS,
+		METH_NOARGS,
 		_mysql_ConnectionObject_errno__doc__
 	},
 	{
 		"field_count",
 		(PyCFunction)_mysql_ConnectionObject_field_count,
-		METH_VARARGS,
+		METH_NOARGS,
 		_mysql_ConnectionObject_field_count__doc__
 	}, 
 	{
 		"get_host_info",
 		(PyCFunction)_mysql_ConnectionObject_get_host_info,
-		METH_VARARGS,
+		METH_NOARGS,
 		_mysql_ConnectionObject_get_host_info__doc__
 	},
 	{
 		"get_proto_info",
 		(PyCFunction)_mysql_ConnectionObject_get_proto_info,
-		METH_VARARGS,
+		METH_NOARGS,
 		_mysql_ConnectionObject_get_proto_info__doc__
 	},
 	{
 		"get_server_info",
 		(PyCFunction)_mysql_ConnectionObject_get_server_info,
-		METH_VARARGS,
+		METH_NOARGS,
 		_mysql_ConnectionObject_get_server_info__doc__
 	},
 	{
 		"info",
 		(PyCFunction)_mysql_ConnectionObject_info,
-		METH_VARARGS,
+		METH_NOARGS,
 		_mysql_ConnectionObject_info__doc__
 	},
 	{
 		"insert_id",
 		(PyCFunction)_mysql_ConnectionObject_insert_id,
-		METH_VARARGS,
+		METH_NOARGS,
 		_mysql_ConnectionObject_insert_id__doc__
 	},
 	{
@@ -2511,19 +2435,19 @@ static PyMethodDef _mysql_ConnectionObject_methods[] = {
 	{
 		"shutdown",
 		(PyCFunction)_mysql_ConnectionObject_shutdown,
-		METH_VARARGS,
+		METH_NOARGS,
 		_mysql_ConnectionObject_shutdown__doc__
 	},
 	{
 		"stat",
 		(PyCFunction)_mysql_ConnectionObject_stat,
-		METH_VARARGS,
+		METH_NOARGS,
 		_mysql_ConnectionObject_stat__doc__
 	},
 	{
 		"store_result",
 		(PyCFunction)_mysql_ConnectionObject_store_result,
-		METH_VARARGS,
+		METH_NOARGS,
 		_mysql_ConnectionObject_store_result__doc__
 	},
 	{
@@ -2534,13 +2458,13 @@ static PyMethodDef _mysql_ConnectionObject_methods[] = {
 	{
 		"thread_id",
 		(PyCFunction)_mysql_ConnectionObject_thread_id,
-		METH_VARARGS,
+		METH_NOARGS,
 		_mysql_ConnectionObject_thread_id__doc__
 	},
 	{
 		"use_result",
 		(PyCFunction)_mysql_ConnectionObject_use_result,
-		METH_VARARGS,
+		METH_NOARGS,
 		_mysql_ConnectionObject_use_result__doc__
 	},
 	{NULL,              NULL} /* sentinel */
@@ -2566,7 +2490,7 @@ static struct PyMemberDef _mysql_ConnectionObject_memberlist[] = {
 		T_UINT,
 		offsetof(_mysql_ConnectionObject,connection.server_capabilities),
 		READONLY,
-		"Capabilites of server; consult MySQLdb.constants.CLIENT"
+		"Capabilities of server; consult MySQLdb.constants.CLIENT"
 	},
 	{
 		"port",
@@ -2601,13 +2525,13 @@ static PyMethodDef _mysql_ResultObject_methods[] = {
 	{
 		"row_tell",
 		(PyCFunction)_mysql_ResultObject_row_tell,
-		METH_VARARGS,
+		METH_NOARGS,
 		_mysql_ResultObject_row_tell__doc__
 	},
 	{
 		"describe",
 		(PyCFunction)_mysql_ResultObject_describe,
-		METH_VARARGS,
+		METH_NOARGS,
 		_mysql_ResultObject_describe__doc__
 	},
 	{
@@ -2619,19 +2543,19 @@ static PyMethodDef _mysql_ResultObject_methods[] = {
 	{
 		"field_flags",
 		(PyCFunction)_mysql_ResultObject_field_flags,
-		METH_VARARGS,
+		METH_NOARGS,
 		_mysql_ResultObject_field_flags__doc__
 	},
 	{
 		"num_fields",
 		(PyCFunction)_mysql_ResultObject_num_fields,
-		METH_VARARGS,
+		METH_NOARGS,
 		_mysql_ResultObject_num_fields__doc__
 	},
 	{
 		"num_rows",
 		(PyCFunction)_mysql_ResultObject_num_rows,
-		METH_VARARGS,
+		METH_NOARGS,
 		_mysql_ResultObject_num_rows__doc__
 	},
 	{NULL,              NULL} /* sentinel */
@@ -2645,6 +2569,13 @@ static struct PyMemberDef _mysql_ResultObject_memberlist[] = {
 		READONLY,
 		"Type conversion mapping"
 	},
+	{
+		"has_next",
+		T_BOOL,
+		offsetof(_mysql_ResultObject, has_next),
+		READONLY,
+		"Has next result"
+	},
 	{NULL} /* Sentinel */
 };
 
@@ -2662,7 +2593,7 @@ _mysql_ConnectionObject_getattro(
 	if (strcmp(cname, "closed") == 0)
 		return PyInt_FromLong((long)!(self->open));
 
-	return PyObject_GenericGetAttr(self, name);
+	return PyObject_GenericGetAttr((PyObject *)self, name);
 }
 
 static int
@@ -2676,7 +2607,7 @@ _mysql_ConnectionObject_setattro(
 				"can't delete connection attributes");
 		return -1;
 	}
-	return PyObject_GenericSetAttr(self, name, v);
+	return PyObject_GenericSetAttr((PyObject *)self, name, v);
 }
 
 static int
@@ -2690,16 +2621,11 @@ _mysql_ResultObject_setattro(
 				"can't delete connection attributes");
 		return -1;
 	}
-	return PyObject_GenericSetAttr(self, name, v);
+	return PyObject_GenericSetAttr((PyObject *)self, name, v);
 }
 
 PyTypeObject _mysql_ConnectionObject_Type = {
-#ifdef IS_PY3K
     PyVarObject_HEAD_INIT(NULL, 0)
-#else
-	PyObject_HEAD_INIT(NULL)
-	0,
-#endif
 	"_mysql.connection", /* (char *)tp_name For printing */
 	sizeof(_mysql_ConnectionObject),
 	0,
@@ -2709,44 +2635,44 @@ PyTypeObject _mysql_ConnectionObject_Type = {
 	0, /* tp_setattr */
 	0, /*tp_compare*/
 	(reprfunc)_mysql_ConnectionObject_repr, /* tp_repr */
-	
+
 	/* Method suites for standard classes */
-	
+
 	0, /* (PyNumberMethods *) tp_as_number */
 	0, /* (PySequenceMethods *) tp_as_sequence */
 	0, /* (PyMappingMethods *) tp_as_mapping */
-	
+
 	/* More standard operations (here for binary compatibility) */
-	
+
 	0, /* (hashfunc) tp_hash */
 	0, /* (ternaryfunc) tp_call */
 	0, /* (reprfunc) tp_str */
 	(getattrofunc)_mysql_ConnectionObject_getattro, /* tp_getattro */
 	(setattrofunc)_mysql_ConnectionObject_setattro, /* tp_setattro */
-	
+
 	/* Functions to access object as input/output buffer */
 	0, /* (PyBufferProcs *) tp_as_buffer */
-	
+
 	/* (tp_flags) Flags to define presence of optional/expanded features */
 	Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC | Py_TPFLAGS_BASETYPE,
 	_mysql_connect__doc__, /* (char *) tp_doc Documentation string */
 
 	/* call function for all accessible objects */
 	(traverseproc) _mysql_ConnectionObject_traverse, /* tp_traverse */
-	
+
 	/* delete references to contained objects */
 	(inquiry) _mysql_ConnectionObject_clear, /* tp_clear */
 
 	/* rich comparisons */
 	0, /* (richcmpfunc) tp_richcompare */
-	
+
 	/* weak reference enabler */
 	0, /* (long) tp_weaklistoffset */
 
 	/* Iterators */
 	0, /* (getiterfunc) tp_iter */
 	0, /* (iternextfunc) tp_iternext */
-	
+
 	/* Attribute descriptor and subclassing stuff */
 	(struct PyMethodDef *)_mysql_ConnectionObject_methods, /* tp_methods */
 	(struct PyMemberDef *)_mysql_ConnectionObject_memberlist, /* tp_members */
@@ -2766,12 +2692,7 @@ PyTypeObject _mysql_ConnectionObject_Type = {
 } ;
 
 PyTypeObject _mysql_ResultObject_Type = {
-#ifdef IS_PY3K
     PyVarObject_HEAD_INIT(NULL, 0)
-#else
-	PyObject_HEAD_INIT(NULL)
-	0,
-#endif
 	"_mysql.result",
 	sizeof(_mysql_ResultObject),
 	0,
@@ -2781,45 +2702,45 @@ PyTypeObject _mysql_ResultObject_Type = {
 	0, /* tp_setattr */
 	0, /*tp_compare*/
 	(reprfunc)_mysql_ResultObject_repr, /* tp_repr */
-	
+
 	/* Method suites for standard classes */
-	
+
 	0, /* (PyNumberMethods *) tp_as_number */
 	0, /* (PySequenceMethods *) tp_as_sequence */
 	0, /* (PyMappingMethods *) tp_as_mapping */
-	
+
 	/* More standard operations (here for binary compatibility) */
-	
+
 	0, /* (hashfunc) tp_hash */
 	0, /* (ternaryfunc) tp_call */
 	0, /* (reprfunc) tp_str */
 	(getattrofunc)PyObject_GenericGetAttr, /* tp_getattro */
 	(setattrofunc)_mysql_ResultObject_setattro, /* tp_setattr */
-	
+
 	/* Functions to access object as input/output buffer */
 	0, /* (PyBufferProcs *) tp_as_buffer */
-	
+
 	/* Flags to define presence of optional/expanded features */
 	Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC | Py_TPFLAGS_BASETYPE,
-	
+
 	_mysql_ResultObject__doc__, /* (char *) tp_doc Documentation string */
 
 	/* call function for all accessible objects */
 	(traverseproc) _mysql_ResultObject_traverse, /* tp_traverse */
-	
+
 	/* delete references to contained objects */
 	(inquiry) _mysql_ResultObject_clear, /* tp_clear */
 
 	/* rich comparisons */
 	0, /* (richcmpfunc) tp_richcompare */
-	
+
 	/* weak reference enabler */
 	0, /* (long) tp_weaklistoffset */
 
 	/* Iterators */
 	0, /* (getiterfunc) tp_iter */
 	0, /* (iternextfunc) tp_iternext */
-	
+
 	/* Attribute descriptor and subclassing stuff */
 	(struct PyMethodDef *) _mysql_ResultObject_methods, /* tp_methods */
 	(struct PyMemberDef *) _mysql_ResultObject_memberlist, /*tp_members */
@@ -2859,12 +2780,14 @@ _mysql_methods[] = {
 		_mysql_escape__doc__
 	},
 	{
+		// deprecated.
 		"escape_sequence",
 		(PyCFunction)_mysql_escape_sequence,
 		METH_VARARGS,
 		_mysql_escape_sequence__doc__
 	},
 	{
+		// deprecated.
 		"escape_dict",
 		(PyCFunction)_mysql_escape_dict,
 		METH_VARARGS,
@@ -2885,17 +2808,15 @@ _mysql_methods[] = {
 	{
 		"get_client_info",
 		(PyCFunction)_mysql_get_client_info,
-		METH_VARARGS,
+		METH_NOARGS,
 		_mysql_get_client_info__doc__
 	},
-#if MYSQL_VERSION_ID >= 32314
 	{
 		"thread_safe",
 		(PyCFunction)_mysql_thread_safe,
-		METH_VARARGS,
+		METH_NOARGS,
 		_mysql_thread_safe__doc__
 	},
-#endif
 	{
 		"server_init",
 		(PyCFunction)_mysql_server_init,
@@ -2918,13 +2839,11 @@ _mysql_NewException(
 	char *name)
 {
 	PyObject *e;
-
 	if (!(e = PyDict_GetItemString(edict, name)))
 		return NULL;
-	if (PyDict_SetItemString(dict, name, e)) return NULL;
-#ifdef PYPY_VERSION
+	if (PyDict_SetItemString(dict, name, e))
+		return NULL;
 	Py_INCREF(e);
-#endif
 	return e;
 }
 
